// stillcapture.cpp : Defines the entry point for the console application.
//

#include <windows.h>
#include <atlbase.h>
#include <streams.h>
#include <dmodshow.h>
#include <dmoreg.h>
#include <wmcodecids.h>
#include <cs.h>
#include <csmedia.h>

#include "../common/CPropertyBag.h"
#include "../common/utils.h"

int _tmain(int argc, _TCHAR* argv[])
{
    HRESULT                         hr;
    TCHAR*                          msg=NULL;
    CComPtr<ICaptureGraphBuilder2>  pCaptureGraphBuilder;
    CComPtr<IMediaEvent>            pMediaEvent;
    CComPtr<IAMVideoControl>        pVideoControl;
    CComPtr<IGraphBuilder>          pFilterGraph;
    CComPtr<IBaseFilter>            pVideoCaptureFilter;
    CComPtr<IBaseFilter>            pVideoEncoder;
    CComPtr<IBaseFilter>            pAsfWriter;
    CComPtr<IBaseFilter>            pStillColourFilter;
    CComPtr<IBaseFilter>            pPreviewColourFilter;
    CComPtr<IBaseFilter>            pImageSinkFilter;
	CComPtr<IBaseFilter>            pVideoRenderer;
    CComPtr<IFileSinkFilter2>       pStillFileSink;
    CComPtr<IFileSinkFilter2>       pAsfFileSink;
    CComPtr<IMediaControl>          pMediaControl;
    CComPtr<IPin>                   pCapturePin;
    CComPtr<IAMStreamConfig>        pCaptureStreamConfig;
    CComPtr<IPin>                   pStillPin;
    CComPtr<IAMStreamConfig>        pStillStreamConfig;
    CComPtr<IPin>                   pPreviewPin;
    CComPtr<IAMStreamConfig>        pPreviewStreamConfig;
    CComPtr<IPersistPropertyBag>    pPropertyBag;
    CComPtr<IDMOWrapperFilter>      pVideoWrapperFilter;
    CComPtr<IMediaSeeking>          pMediaSeeking;
    CComPtr<IVideoWindow>			pVideoWindow;
    CComVariant                     varCamName;
    CPropertyBag                    PropBag;
    DWORD                           timeout=30;
    TCHAR                           capturefilename[MAX_PATH+1];
    TCHAR                           stillfilename[MAX_PATH+1];
    TCHAR*                          captureres;
    TCHAR*                          previewres;
    BOOL                            enablepreview=FALSE;
    BOOL                            enablecapture=FALSE;
    BOOL                            enablestill=FALSE;

    if ((argc==1)||(argc>8))
    {
        _tprintf(TEXT("usage: camera -s/c <capture res> [filename] -p <preview res> - t [seconds]\r\n"));
        _tprintf(TEXT("-s still capture\r\n"));
        _tprintf(TEXT("-c video capture\r\n"));
        _tprintf(TEXT("-p show preview\r\n"));
		_tprintf(TEXT("[filename] is optional and must be the full path of saved video or image, if no path is specified \\<capture res>.ASF or \\<capture res>.png will be used.\r\n"));        
        _tprintf(TEXT("Video capture/preview is going to be run for [seconds], or 30 seconds if no value is provided.\r\n"));
    }

    *capturefilename=0;
    *stillfilename=0;

    captureres=NULL;
    previewres=NULL;

    // parse cmdline
    for (int i=1;i<argc;i++)
    {
        if (_tcsicmp(TEXT("-s"),argv[i])==0)
        {
            // capture still image
            if (enablecapture)
            {
                msg=TEXT("s and c parameters are mutually exclusive.\r\n");
                goto error;
            }

            if (i==argc-1)
            {
                msg=TEXT("Please specify resolution.\r\n");
                goto error;
            }

            enablestill=TRUE;

            i++;

            captureres=argv[i];

            // we will use defaults
            if (i==argc-1)
                continue;


            //check if it's a switch, otherwise it's filename
            if (argv[i+1][0]==TEXT('-'))
                continue;

            i++;

            _tcscpy_s(stillfilename,MAX_PATH,argv[i]);
        }
        else if (_tcsicmp(TEXT("-c"),argv[i])==0)
        {
            // capture video
            if (enablestill)
            {
                msg=TEXT("s and c parameters are mutually exclusive.\r\n");
                goto error;
            }

            if (i==argc-1)
            {
                msg=TEXT("Please specify resolution.\r\n");
                goto error;
            }

            enablecapture=TRUE;

            i++;

            captureres=argv[i];

            // we will use defaults
            if (i==argc-1)
                continue;


            //check if it's a switch, otherwise it's filename
            if (argv[i+1][0]==TEXT('-'))
                continue;

            i++;

            _tcscpy_s(capturefilename,MAX_PATH,argv[i]);
        }
        else if (_tcsicmp(TEXT("-p"),argv[i])==0)
        {
            // capture preview
            if (i==argc-1)
            {
                msg=TEXT("Please specify resolution.\r\n");
                goto error;
            }

            enablepreview=TRUE;

            i++;

            previewres=argv[i];
        }
        else if (_tcsicmp(TEXT("-t"),argv[i])==0)
        {
            if (i==argc-1)
            {
                msg=TEXT("Please specify a timeout.\r\n");
                goto error;
            }

            i++;

            TCHAR* endp;

            timeout=_tcstoul(argv[i],&endp,10);

            if ((timeout==0)||(*endp!=0))
            {
                msg=TEXT("Invalid timeout.\r\n");
                goto error;
            }
        }
    }

    if ((enablecapture)&&(*capturefilename==0))
    {
        _tcscpy_s(capturefilename,MAX_PATH,TEXT("\\"));
        _tcscat_s(capturefilename,MAX_PATH,captureres);
        _tcscat_s(capturefilename,MAX_PATH,TEXT(".ASF"));
    }

    if ((enablestill)&&(*stillfilename==0))
    {
        _tcscpy_s(stillfilename,MAX_PATH,TEXT("\\"));
        _tcscat_s(stillfilename,MAX_PATH,captureres);
        _tcscat_s(stillfilename,MAX_PATH,TEXT(".PNG"));
    }

    // initialize COM
    if (FAILED(hr=CoInitializeEx(NULL,COINIT_MULTITHREADED)))
    {
        msg=TEXT("CoInitialize failed.");
        goto error;
    }

    // create an instance of the video capture filter
    if (FAILED(hr=pVideoCaptureFilter.CoCreateInstance(CLSID_VideoCapture)))
    {
        msg=TEXT("Error creating capture filter.");
        goto error;
    }

    if (FAILED(hr=pVideoCaptureFilter.QueryInterface(&pVideoControl)))
    {
        msg=TEXT("Error getting capture filter control interface.");
        goto error;
    }

    // match it with a video capture driver
    if (FAILED(hr=pVideoCaptureFilter.QueryInterface(&pPropertyBag)))
    {
        msg=TEXT("Error getting capture filter interface.");
        goto error;
    }   

    TCHAR cameraName[_MAX_PATH+1];

    if (FAILED(hr=GetCameraDriver(cameraName)))
    {
        msg=TEXT("Error searching for camera driver.");
        goto error;
    }

    varCamName=cameraName;

    if (FAILED(hr=PropBag.Write(TEXT("VCapName"),&varCamName)))
    {
        msg=TEXT("Error configuring property bag.");
        goto error;
    }

    if (FAILED(hr=pPropertyBag->Load(&PropBag,NULL)))
    {
        msg=TEXT("Error loading property bag.");
        goto error;
    }

	if (enablestill)
	{
	    // create color converter
	    if (FAILED(hr=pStillColourFilter.CoCreateInstance(CLSID_Colour)))
	    {
	        msg=TEXT("Error creating capture filter.");
	        goto error;
	    }

	    // create image sink
	    if (FAILED(hr=pImageSinkFilter.CoCreateInstance(CLSID_IMGSinkFilter)))
	    {
	        msg=TEXT("Error creating image sink.");
	        goto error;
	    }
	
	    if (FAILED(hr=pImageSinkFilter.QueryInterface(&pStillFileSink)))
	    {
	        msg=TEXT("Error getting image sink interface.");
	        goto error;
	    }
	
	    if (FAILED(hr=pStillFileSink->SetMode(AM_FILE_OVERWRITE)))
	    {
	        msg=TEXT("Error getting image sink interface.");
	        goto error;
	    }
	
        if (FAILED(hr=pStillFileSink->SetFileName(stillfilename,NULL)))
        {
            msg=TEXT("Error configuring output filename.");
            goto error;
        }
	}

	if (enablecapture)
	{	
	    // create DMO video encoder    
	    if (FAILED(hr=pVideoEncoder.CoCreateInstance(CLSID_DMOWrapperFilter)))
	    {
	        msg=TEXT("Error creating video encoder.");
	        goto error;
	    }
	    
	    if (FAILED(hr=pVideoEncoder.QueryInterface(&pVideoWrapperFilter)))
	    {
	        msg=TEXT("Error getting video wrapper interface.");
	        goto error;
	    }
	
	    if (FAILED(hr=pVideoWrapperFilter->Init(CLSID_CWMV9EncMediaObject,DMOCATEGORY_VIDEO_ENCODER)))
	    {
	        msg=TEXT("Error initializing video wrapper.");
	        goto error;
	    }
	
	    // create asf writer
	    if (FAILED(hr=pAsfWriter.CoCreateInstance(CLSID_ASFWriter)))
	    {
	        msg=TEXT("Error creating video renderer.");
	        goto error;
	    }
	
	    if (FAILED(hr=pAsfWriter.QueryInterface(&pAsfFileSink)))
	    {
	        msg=TEXT("Error getting image sink interface.");
	        goto error;
	    }
	
	    if (FAILED(hr=pAsfFileSink->SetMode(AM_FILE_OVERWRITE)))
	    {
	        msg=TEXT("Error getting image sink interface.");
	        goto error;
	    }
	
	    if (FAILED(hr=pAsfFileSink->SetFileName(capturefilename,NULL)))
	    {
	        msg=TEXT("Error configuring output filename.");
	        goto error;
	    }
	}

	if (enablepreview)
	{	
	    // create color converter
	    if (FAILED(hr=pPreviewColourFilter.CoCreateInstance(CLSID_Colour)))
	    {
	        msg=TEXT("Error creating capture filter.");
	        goto error;
	    }

	    // create video renderer
	    if (FAILED(hr=pVideoRenderer.CoCreateInstance(CLSID_VideoMixingRenderer)))
	    {
	        msg=TEXT("Error creating video renderer.");
	        goto error;
	    }
	}

    // create the graph builder and filter graph and add video capture filter and image sink filters to it   
    if (FAILED(hr=pCaptureGraphBuilder.CoCreateInstance(CLSID_CaptureGraphBuilder)))
    {
        msg=TEXT("Error creating graph builder.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph.CoCreateInstance(CLSID_FilterGraph)))
    {
        msg=TEXT("Error creating filter graph.");
        goto error;
    }

	if (enablecapture || enablestill)
	{
	    if (FAILED(hr=pFilterGraph.QueryInterface(&pMediaEvent)))
	    {
	        msg=TEXT("Error getting media event interface.");
	        goto error;
	    }    
	}
	    
	if (enablecapture)
	{	
	    if (FAILED(hr=pFilterGraph.QueryInterface(&pMediaSeeking)))
	    {
	        msg=TEXT("Error getting media seeking interface.");
	        goto error;
	    }    
	}

	if (enablepreview)
	{
	    if (FAILED(hr=pFilterGraph.QueryInterface(&pVideoWindow)))
    	{
        	msg=TEXT("Error getting video window interface.");
        	goto error;
    	}    
	}


    if (FAILED(hr=pCaptureGraphBuilder->SetFiltergraph(pFilterGraph)))
    {
        msg=TEXT("Error adding filter graph to graph builder.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph->AddFilter(pVideoCaptureFilter,TEXT("Video Capture"))))
    {
        msg=TEXT("Error adding video capture filter to filter graph.");
        goto error;
    }

	if (enablestill)
	{
	    if (FAILED(hr=pFilterGraph->AddFilter(pStillColourFilter,TEXT("Colour converter still images"))))
	    {
	        msg=TEXT("Error adding color converter filter to filter graph.");
	        goto error;
	    }
	
	    if (FAILED(hr=pFilterGraph->AddFilter(pImageSinkFilter,TEXT("Image sink"))))
	    {
	        msg=TEXT("Error adding image sink to filter graph.");
	        goto error;
	    }
	}

	if (enablepreview)
	{
		if (FAILED(hr=pFilterGraph->AddFilter(pPreviewColourFilter,TEXT("Colour converter preview"))))
	    {
	        msg=TEXT("Error adding video capture filter to filter graph.");
	        goto error;
	    }
	
	    if (FAILED(hr=pFilterGraph->AddFilter(pVideoRenderer,TEXT("Video renderer"))))
	    {
	        msg=TEXT("Error adding image sink to filter graph.");
	        goto error;
	    }
	}

	if (enablecapture)
	{
	    if (FAILED(hr=pFilterGraph->AddFilter(pVideoEncoder,TEXT("Video encoder"))))
	    {
	        msg=TEXT("Error adding video encoder filter to filter graph.");
	        goto error;
	    }
	
	    if (FAILED(hr=pFilterGraph->AddFilter(pAsfWriter,TEXT("ASF writer"))))
	    {
	        msg=TEXT("Error adding ASF writer to filter graph.");
	        goto error;
	    }
	}

    if (argc==1)
    {           
        // we need to configure the resolution
        if (FAILED(hr=pCaptureGraphBuilder->FindPin(pVideoCaptureFilter,PINDIR_OUTPUT,&PIN_CATEGORY_CAPTURE,&MEDIATYPE_Video,FALSE,0,&pCapturePin)))
        {
            msg=TEXT("Error accessing capture pin.");
            goto error;
        }

        if (FAILED(hr=pCaptureGraphBuilder->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, pVideoCaptureFilter, IID_IAMStreamConfig, (void **) &pCaptureStreamConfig)))
        {
            msg=TEXT("Error accessing configuration interface.");
            goto error;
        }

        _tprintf(TEXT("Resolutions for capture:\r\n"));

        if (FAILED(hr=EnumConfigs(pCaptureStreamConfig,NULL,ConfigComparePrint,NULL)))
        {
            msg=TEXT("Error enumerating configurations.");
            goto error;
        }

        if (FAILED(hr=pCaptureGraphBuilder->FindPin(pVideoCaptureFilter,PINDIR_OUTPUT,&PIN_CATEGORY_STILL,&MEDIATYPE_Video,FALSE,0,&pStillPin)))
        {
            msg=TEXT("Error accessing still capture pin.");
            goto error;
        }

        if (FAILED(hr=pCaptureGraphBuilder->FindInterface(&PIN_CATEGORY_STILL, &MEDIATYPE_Video, pVideoCaptureFilter, IID_IAMStreamConfig, (void **) &pStillStreamConfig)))
        {
            msg=TEXT("Error accessing configuration interface.");
            goto error;
        }

        _tprintf(TEXT("Resolutions for still images:\r\n"));

        if (FAILED(hr=EnumConfigs(pStillStreamConfig,NULL,ConfigComparePrint,NULL)))
        {
            msg=TEXT("Error enumerating configurations.");
            goto error;
        }

        if (FAILED(hr=pCaptureGraphBuilder->FindPin(pVideoCaptureFilter,PINDIR_OUTPUT,&PIN_CATEGORY_PREVIEW,&MEDIATYPE_Video,FALSE,0,&pPreviewPin)))
        {
            msg=TEXT("Error accessing preview pin.");
            goto error;
        }

        if (FAILED(hr=pCaptureGraphBuilder->FindInterface(&PIN_CATEGORY_PREVIEW, &MEDIATYPE_Video, pVideoCaptureFilter, IID_IAMStreamConfig, (void **) &pPreviewStreamConfig)))
        {
            msg=TEXT("Error accessing configuration interface.");
            goto error;
        }

        _tprintf(TEXT("Resolutions for preview:\r\n"));

        if (FAILED(hr=EnumConfigs(pPreviewStreamConfig,NULL,ConfigComparePrint,NULL)))
        {
            msg=TEXT("Error enumerating configurations.");
            goto error;
        }

        return S_OK;
    }


    AM_MEDIA_TYPE* config=NULL;
    
    // we need to configure the resolution
    if (enablecapture)
    {
        if (FAILED(hr=pCaptureGraphBuilder->FindPin(pVideoCaptureFilter,PINDIR_OUTPUT,&PIN_CATEGORY_CAPTURE,&MEDIATYPE_Video,FALSE,0,&pCapturePin)))
        {
            msg=TEXT("Error accessing capture pin.");
            goto error;
        }

        if (FAILED(hr=pCaptureGraphBuilder->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, pVideoCaptureFilter, IID_IAMStreamConfig, (void **) &pCaptureStreamConfig)))
        {
            msg=TEXT("Error accessing configuration interface.");
            goto error;
        }

        if (FAILED(hr=EnumConfigs(pCaptureStreamConfig,&config,ConfigCompareString,captureres)))
        {
            msg=TEXT("Error enumerating configurations.");
            goto error;
        }

        if (!config)
        {
            msg=TEXT("No compatible resolution found.");
            goto error;
        }

        if (FAILED(hr=pCaptureStreamConfig->SetFormat(config)))
        {
            msg=TEXT("Can't configure selected video format.");
            goto error;
        }
                            
        DeleteMediaType(config);

        // renders the stream
        if (FAILED(hr=pCaptureGraphBuilder->RenderStream(&PIN_CATEGORY_CAPTURE,&MEDIATYPE_Video,pVideoCaptureFilter,pVideoEncoder,pAsfWriter)))
        {
            msg=TEXT("Error rendering stream.");
            goto error;
        }

        // block capture on the video pin    
        if (FAILED(hr=pCaptureGraphBuilder->ControlStream(&PIN_CATEGORY_CAPTURE,&MEDIATYPE_Video,pVideoCaptureFilter,0,0,0,0)))
        {
            msg=TEXT("Error controlling stream.");
            goto error;
        }

    }
    else if (enablestill)
    {
        // we need to configure the resolution
        if (FAILED(hr=pCaptureGraphBuilder->FindPin(pVideoCaptureFilter,PINDIR_OUTPUT,&PIN_CATEGORY_STILL,&MEDIATYPE_Video,FALSE,0,&pStillPin)))
        {
            msg=TEXT("Error accessing still capture pin.");
            goto error;
        }

        if (FAILED(hr=pCaptureGraphBuilder->FindInterface(&PIN_CATEGORY_STILL, &MEDIATYPE_Video, pVideoCaptureFilter, IID_IAMStreamConfig, (void **) &pStillStreamConfig)))
        {
            msg=TEXT("Error accessing configuration interface.");
            goto error;
        }

        if (FAILED(hr=EnumConfigs(pStillStreamConfig,&config,ConfigCompareString,captureres)))
        {
            msg=TEXT("Error enumerating configurations.");
            goto error;
        }

        if (!config)
        {
            msg=TEXT("No compatible resolution found.");
            goto error;
        }

        if (FAILED(hr=pStillStreamConfig->SetFormat(config)))
        {
            msg=TEXT("Can't configure selected video format.");
            goto error;
        }

        DeleteMediaType(config);

        // renders the stream
        if (FAILED(hr=pCaptureGraphBuilder->RenderStream(&PIN_CATEGORY_STILL,&MEDIATYPE_Video,pVideoCaptureFilter,pStillColourFilter,pImageSinkFilter)))
        {
            msg=TEXT("Error rendering stream.");
            goto error;
        }
    }

    if (enablepreview)
    {
        if (FAILED(hr=pCaptureGraphBuilder->FindPin(pVideoCaptureFilter,PINDIR_OUTPUT,&PIN_CATEGORY_PREVIEW,&MEDIATYPE_Video,FALSE,0,&pPreviewPin)))
        {
            msg=TEXT("Error accessing preview pin.");
            goto error;
        }

        if (FAILED(hr=pCaptureGraphBuilder->FindInterface(&PIN_CATEGORY_PREVIEW, &MEDIATYPE_Video, pVideoCaptureFilter, IID_IAMStreamConfig, (void **) &pPreviewStreamConfig)))
        {
            msg=TEXT("Error accessing configuration interface.");
            goto error;
        }

        if (FAILED(hr=EnumConfigs(pPreviewStreamConfig,&config,ConfigCompareString,previewres)))
        {
            msg=TEXT("Error enumerating configurations.");
            goto error;
        }

        if (!config)
        {
            msg=TEXT("No compatible resolution found.");
            goto error;
        }

        LONG videoWidth=abs(((VIDEOINFOHEADER*)config->pbFormat)->bmiHeader.biWidth);
        LONG videoHeight=abs(((VIDEOINFOHEADER*)config->pbFormat)->bmiHeader.biHeight);

        if (FAILED(hr=pPreviewStreamConfig->SetFormat(config)))
        {
            msg=TEXT("Can't configure selected video format.");
            goto error;
        }

        // configure the video window
        RECT screen;
        RECT video;

        GetWindowRect(GetDesktopWindow(),&screen);

        video.left=max(0,(screen.right-videoWidth)/2);
        video.top=max(0,(screen.bottom-videoHeight)/2);
        video.right=min(screen.right,(screen.right+videoWidth)/2);
        video.bottom=min(screen.bottom,(screen.bottom+videoHeight)/2);

        if ((videoWidth>screen.right)||(videoHeight>screen.bottom))
        {
            _tprintf(TEXT("Capture resolution is higher than video resolution, video will be stretched, frame rate may decrease and CPU usage will increase.\r\n"));
        }

        // renders the stream
        if (FAILED(hr=pCaptureGraphBuilder->RenderStream(&PIN_CATEGORY_PREVIEW,&MEDIATYPE_Video,pVideoCaptureFilter,pPreviewColourFilter,pVideoRenderer)))
        {
            msg=TEXT("Error rendering stream.");
            goto error;
        }
        
        if (FAILED(hr=pVideoWindow->SetWindowPosition(video.left,video.top,video.right-video.left,video.bottom-video.top)))
        {
            msg=TEXT("Can't configure video window position.");
            goto error;
        }

        if (FAILED(hr=pVideoWindow->put_WindowStyleEx(WS_EX_TOPMOST)))
        {
            msg=TEXT("Can't configure video window position.");
            goto error;
        }

        DeleteMediaType(config);
    }

    // now we need to start the graph
    if (FAILED(hr=pFilterGraph.QueryInterface(&pMediaControl)))
    {
        msg=TEXT("Error accessing media control interface.");
        goto error;
    }

    if (FAILED(hr=pMediaControl->Run()))
    {
        msg=TEXT("Error running the graph.");
        goto error;
    }

    // preview is already running at this point
    if (enablepreview)
    {
        _tprintf(TEXT("Activating preview for %d seconds.\r\n"),timeout);

        if (!enablecapture)
        {
            Sleep(timeout*1000);
        }
    }

    if (enablecapture)
    {
        _tprintf(TEXT("Activating capture for %d seconds.\r\n"),timeout);

        REFERENCE_TIME rtstart=0;
        REFERENCE_TIME rtend=MAXLONGLONG;

        // start capture on the video pin    
        if (FAILED(hr=pCaptureGraphBuilder->ControlStream(&PIN_CATEGORY_CAPTURE,&MEDIATYPE_Video,pVideoCaptureFilter,&rtstart,&rtend,0,0)))
        {
            msg=TEXT("Error starting capture.");
            goto error;
        }

        Sleep(timeout*1000);

        // stop capture on the video pin    
        if (FAILED(hr=pMediaSeeking->GetCurrentPosition(&rtend)))
        {
            msg=TEXT("Error starting capture.");
            goto error;
        }

        if (FAILED(hr=pCaptureGraphBuilder->ControlStream(&PIN_CATEGORY_CAPTURE,&MEDIATYPE_Video,pVideoCaptureFilter,&rtstart,&rtend,1,2)))
        {
            msg=TEXT("Error starting capture.");
            goto error;
        }

        long code,param1,param2;
        DWORD start=GetTickCount();

        do
        {
            if (FAILED(hr=pMediaEvent->GetEvent(&code,&param1,&param2,10000)))
            {
                msg=TEXT("Error while waiting for event.");
                goto error;
            }

            if (FAILED(hr=pMediaEvent->FreeEventParams(code,param1,param2)))
            {
                msg=TEXT("Error freeing event information.");
                goto error;
            }        
        }
        while ((code!=EC_STREAM_CONTROL_STOPPED)&&(GetTickCount()-start<10000));
    }
    else if (enablestill)
    {
        if (FAILED(hr=pVideoControl->SetMode(pStillPin,VideoControlFlag_Trigger)))
        {
            msg=TEXT("Error triggering capture event.");
            goto error;
        }

        long code,param1,param2;
        DWORD start=GetTickCount();

        do
        {
            if (FAILED(hr=pMediaEvent->GetEvent(&code,&param1,&param2,10000)))
            {
                msg=TEXT("Error while waiting for event.");
                goto error;
            }

            if (FAILED(hr=pMediaEvent->FreeEventParams(code,param1,param2)))
            {
                msg=TEXT("Error freeing event information.");
                goto error;
            }        
        }
        while ((code!=EC_CAP_FILE_COMPLETED)&&(GetTickCount()-start<10000));

        if (code!=EC_CAP_FILE_COMPLETED)
        {
            _tprintf(TEXT("No image has been captured."));
        }
        else
        {
            _tprintf(TEXT("Still image captured.\r\n"));
        }
    }

    if (FAILED(hr=pMediaControl->Stop()))
    {
        msg=TEXT("Error stopping the graph.");
        goto error;
    }

    _tprintf(TEXT("Capture terminated.\r\n"));   
    return hr;

error:

    _tprintf(TEXT("%s\r\nCode: %d(%08x)"),msg,hr,hr);
    return hr;
}

