// stillcapture.cpp : Defines the entry point for the console application.
//

#include <windows.h>
#include <atlbase.h>
#include <streams.h>
#include <dmodshow.h>
#include <dmoreg.h>
#include <wmcodecids.h>
#include <cs.h>
#include <csmedia.h>

#include "../common/utils.h"
#include "../common/CPropertyBag.h"

int _tmain(int argc, _TCHAR* argv[])
{
    HRESULT                         hr;
    TCHAR*                          msg=NULL;
    CComPtr<ICaptureGraphBuilder2>  pCaptureGraphBuilder;
    CComPtr<IMediaEvent>            pMediaEvent;
    CComPtr<IGraphBuilder>          pFilterGraph;
    CComPtr<IBaseFilter>            pVideoCaptureFilter;
    CComPtr<IBaseFilter>            pVideoEncoder;
    CComPtr<IBaseFilter>            pAsfWriter;
    CComPtr<IFileSinkFilter2>       pAsfFileSink;
    CComPtr<IMediaControl>          pMediaControl;
    CComPtr<IPin>                   pCapturePin;
    CComPtr<IAMStreamConfig>        pPreviewStreamConfig;
    CComPtr<IAMStreamConfig>        pCaptureStreamConfig;
    CComPtr<IPersistPropertyBag>    pPropertyBag;
    CComPtr<IDMOWrapperFilter>      pVideoWrapperFilter;
    CComPtr<IMediaSeeking>          pMediaSeeking;
    CComVariant                     varCamName;
    CPropertyBag                    PropBag;
    DWORD                           timeout=30;

    if (argc==4)
    {
        TCHAR* endp;

        timeout=_tcstoul(argv[4],&endp,10);

        if ((timeout==0)&&(*endp!=0))
        {
            _tprintf(TEXT("Invalid timeout.\r\n"));
            return -1;
        }
    }

    if ((argc==1)||(argc>3))
    {
        _tprintf(TEXT("usage: capture <res> [filename] [seconds]\r\n"));
		_tprintf(TEXT("[filename] is optional and must be the full path of saved video, if no path is specified \\<res>.ASF will be used.\r\n"));        
        _tprintf(TEXT("Video capture going to be run for [seconds], or 30 seconds if no value is provided.\r\n"));
        _tprintf(TEXT("<res> is mandatory and can be one of the following values:\r\n"));
    }

    // initialize COM
    if (FAILED(hr=CoInitializeEx(NULL,COINIT_MULTITHREADED)))
    {
        msg=TEXT("CoInitialize failed.");
        goto error;
    }

    // create an instance of the video capture filter
    if (FAILED(hr=pVideoCaptureFilter.CoCreateInstance(CLSID_VideoCapture)))
    {
        msg=TEXT("Error creating capture filter.");
        goto error;
    }

    // match it with a video capture driver
    if (FAILED(hr=pVideoCaptureFilter.QueryInterface(&pPropertyBag)))
    {
        msg=TEXT("Error getting capture filter interface.");
        goto error;
    }   

    TCHAR cameraName[_MAX_PATH+1];

    if (FAILED(hr=GetCameraDriver(cameraName)))
    {
        msg=TEXT("Error searching for camera driver.");
        goto error;
    }

    varCamName=cameraName;

    if (FAILED(hr=PropBag.Write(TEXT("VCapName"),&varCamName)))
    {
        msg=TEXT("Error configuring property bag.");
        goto error;
    }

    if (FAILED(hr=pPropertyBag->Load(&PropBag,NULL)))
    {
        msg=TEXT("Error loading property bag.");
        goto error;
    }

    // create DMO video encoder    
    if (FAILED(hr=pVideoEncoder.CoCreateInstance(CLSID_DMOWrapperFilter)))
    {
        msg=TEXT("Error creating video encoder.");
        goto error;
    }
    
    if (FAILED(hr=pVideoEncoder.QueryInterface(&pVideoWrapperFilter)))
    {
        msg=TEXT("Error getting video wrapper interface.");
        goto error;
    }

    if (FAILED(hr=pVideoWrapperFilter->Init(CLSID_CWMV9EncMediaObject,DMOCATEGORY_VIDEO_ENCODER)))
    {
        msg=TEXT("Error initializing video wrapper.");
        goto error;
    }

    // create video renderer
    if (FAILED(hr=pAsfWriter.CoCreateInstance(CLSID_ASFWriter)))
    {
        msg=TEXT("Error creating video renderer.");
        goto error;
    }

    if (FAILED(hr=pAsfWriter.QueryInterface(&pAsfFileSink)))
    {
        msg=TEXT("Error getting image sink interface.");
        goto error;
    }

    if (FAILED(hr=pAsfFileSink->SetMode(AM_FILE_OVERWRITE)))
    {
        msg=TEXT("Error getting image sink interface.");
        goto error;
    }

    if (argc>=3)
    {
        if (FAILED(hr=pAsfFileSink->SetFileName(argv[2],NULL)))
        {
            msg=TEXT("Error configuring output filename.");
            goto error;
        }
    }
    else
    {
        if (argc>1)
        {
            // otherwise just use resolution
            TCHAR filename[_MAX_PATH+1];

            _tcsncpy_s(filename,_MAX_PATH,argv[1],MAX_PATH);
            _tcsncat_s(filename,_MAX_PATH,TEXT(".ASF"),MAX_PATH);

            if (FAILED(hr=pAsfFileSink->SetFileName(filename,NULL)))
            {
                msg=TEXT("Error configuring output filename.");
                goto error;
            }
        }
    }
    // create the graph builder and filter graph and add video capture filter and image sink filters to it   
    if (FAILED(hr=pCaptureGraphBuilder.CoCreateInstance(CLSID_CaptureGraphBuilder)))
    {
        msg=TEXT("Error creating graph builder.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph.CoCreateInstance(CLSID_FilterGraph)))
    {
        msg=TEXT("Error creating filter graph.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph.QueryInterface(&pMediaEvent)))
    {
        msg=TEXT("Error getting media event interface.");
        goto error;
    }    
    
    if (FAILED(hr=pFilterGraph.QueryInterface(&pMediaSeeking)))
    {
        msg=TEXT("Error getting media seeking interface.");
        goto error;
    }    

    if (FAILED(hr=pCaptureGraphBuilder->SetFiltergraph(pFilterGraph)))
    {
        msg=TEXT("Error adding filter graph to graph builder.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph->AddFilter(pVideoCaptureFilter,TEXT("Video Capture"))))
    {
        msg=TEXT("Error adding video capture filter to filter graph.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph->AddFilter(pVideoEncoder,TEXT("Video encoder"))))
    {
        msg=TEXT("Error adding video encoder filter to filter graph.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph->AddFilter(pAsfWriter,TEXT("ASF writer"))))
    {
        msg=TEXT("Error adding ASF writer to filter graph.");
        goto error;
    }
    
    // we need to configure the resolution
    if (FAILED(hr=pCaptureGraphBuilder->FindPin(pVideoCaptureFilter,PINDIR_OUTPUT,&PIN_CATEGORY_CAPTURE,&MEDIATYPE_Video,FALSE,0,&pCapturePin)))
    {
        msg=TEXT("Error accessing capture pin.");
        goto error;
    }

    if (FAILED(hr=pCaptureGraphBuilder->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, pVideoCaptureFilter, IID_IAMStreamConfig, (void **) &pCaptureStreamConfig)))
    {
        msg=TEXT("Error accessing configuration interface.");
        goto error;
    }

    if (argc==1)
    {           
        if (FAILED(hr=EnumConfigs(pCaptureStreamConfig,NULL,ConfigComparePrint,NULL)))
        {
            msg=TEXT("Error enumerating configurations.");
            goto error;
        }
        // we just needed to print out configurations
        return S_OK;
    }

    AM_MEDIA_TYPE* config=NULL;

    if (FAILED(hr=EnumConfigs(pCaptureStreamConfig,&config,ConfigCompareString,argv[1])))
    {
        msg=TEXT("Error enumerating configurations.");
        goto error;
    }

    if (!config)
    {
        msg=TEXT("No compatible resolution found.");
        goto error;
    }

    if (FAILED(hr=pCaptureStreamConfig->SetFormat(config)))
    {
        msg=TEXT("Can't configure selected video format.");
        goto error;
    }
                        
    DeleteMediaType(config);

    // renders the stream
    if (FAILED(hr=pCaptureGraphBuilder->RenderStream(&PIN_CATEGORY_CAPTURE,&MEDIATYPE_Video,pVideoCaptureFilter,pVideoEncoder,pAsfWriter)))
    {
        msg=TEXT("Error rendering stream.");
        goto error;
    }

    // block capture on the video pin    
    if (FAILED(hr=pCaptureGraphBuilder->ControlStream(&PIN_CATEGORY_CAPTURE,&MEDIATYPE_Video,pVideoCaptureFilter,0,0,0,0)))
    {
        msg=TEXT("Error controlling stream.");
        goto error;
    }

    // now we need to start the graph
    if (FAILED(hr=pFilterGraph.QueryInterface(&pMediaControl)))
    {
        msg=TEXT("Error accessing media control interface.");
        goto error;
    }

    if (FAILED(hr=pMediaControl->Run()))
    {
        msg=TEXT("Error running the graph.");
        goto error;
    }

    _tprintf(TEXT("Activating capture for %d seconds.\r\n"),timeout);

    REFERENCE_TIME rtstart=0;
    REFERENCE_TIME rtend=MAXLONGLONG;

    // start capture on the video pin    
    if (FAILED(hr=pCaptureGraphBuilder->ControlStream(&PIN_CATEGORY_CAPTURE,&MEDIATYPE_Video,pVideoCaptureFilter,&rtstart,&rtend,0,0)))
    {
        msg=TEXT("Error starting capture.");
        goto error;
    }

    Sleep(timeout*1000);

    // stop capture on the video pin    
    if (FAILED(hr=pMediaSeeking->GetCurrentPosition(&rtend)))
    {
        msg=TEXT("Error starting capture.");
        goto error;
    }

    if (FAILED(hr=pCaptureGraphBuilder->ControlStream(&PIN_CATEGORY_CAPTURE,&MEDIATYPE_Video,pVideoCaptureFilter,&rtstart,&rtend,1,2)))
    {
        msg=TEXT("Error starting capture.");
        goto error;
    }

    long code,param1,param2;
    DWORD start=GetTickCount();

    do
    {
        if (FAILED(hr=pMediaEvent->GetEvent(&code,&param1,&param2,10000)))
        {
            msg=TEXT("Error while waiting for event.");
            goto error;
        }

        if (FAILED(hr=pMediaEvent->FreeEventParams(code,param1,param2)))
        {
            msg=TEXT("Error freeing event information.");
            goto error;
        }        
    }
    while ((code!=EC_STREAM_CONTROL_STOPPED)&&(GetTickCount()-start<10000));

    if (FAILED(hr=pMediaControl->Stop()))
    {
        msg=TEXT("Error stopping the graph.");
        goto error;
    }

    _tprintf(TEXT("Capture terminated.\r\n"));   
    return hr;

error:

    _tprintf(TEXT("%s\r\nCode: %d(%08x)"),msg,hr,hr);
    return hr;
}

