// stillcapture.cpp : Defines the entry point for the console application.
//

#include <windows.h>
#include <atlbase.h>
#include <streams.h>
#include <dmodshow.h>
#include <dmoreg.h>
#include <wmcodecids.h>
#include <cs.h>
#include <csmedia.h>

#include "../common/CPropertyBag.h"
#include "../common/utils.h"

int _tmain(int argc, _TCHAR* argv[])
{
    HRESULT                         hr;
    TCHAR*                          msg=NULL;
    CComPtr<ICaptureGraphBuilder2>  pCaptureGraphBuilder;
    CComPtr<IGraphBuilder>          pFilterGraph;
    CComPtr<IBaseFilter>            pVideoCaptureFilter;
    CComPtr<IBaseFilter>            pColourFilter;
    CComPtr<IBaseFilter>            pVideoRenderer;
    CComPtr<IMediaControl>          pMediaControl;
    CComPtr<IPin>                   pPreviewPin;
    CComPtr<IAMStreamConfig>        pPreviewStreamConfig;
    CComPtr<IPersistPropertyBag>    pPropertyBag;
    CComPtr<IVideoWindow>			pVideoWindow;
    CComVariant                     varCamName;
    CPropertyBag                    PropBag;
    DWORD                           timeout=30;

    if (argc==3)
    {
        TCHAR* endp;

        timeout=_tcstoul(argv[2],&endp,10);

        if ((timeout==0)&&(*endp!=0))
        {
            _tprintf(TEXT("Invalid timeout.\r\n"));
            return -1;
        }
    }

    if ((argc==1)||(argc>3))
    {
        _tprintf(TEXT("usage: preview <res> [seconds]\r\n"));
        _tprintf(TEXT("Preview is going to be active for [seconds], or 30 seconds if no value is provided.\r\n"));
        _tprintf(TEXT("<res> is mandatory and can be one of the following values:\r\n"));
    }

    // initialize COM
    if (FAILED(hr=CoInitializeEx(NULL,COINIT_MULTITHREADED)))
    {
        msg=TEXT("CoInitialize failed.");
        goto error;
    }


    // create an instance of the video capture filter
    if (FAILED(hr=pVideoCaptureFilter.CoCreateInstance(CLSID_VideoCapture)))
    {
        msg=TEXT("Error creating capture filter.");
        goto error;
    }

    // match it with a video capture driver
    if (FAILED(hr=pVideoCaptureFilter.QueryInterface(&pPropertyBag)))
    {
        msg=TEXT("Error getting capture filter interface.");
        goto error;
    }   

    TCHAR cameraName[_MAX_PATH+1];

    if (FAILED(hr=GetCameraDriver(cameraName)))
    {
        msg=TEXT("Error searching for camera driver.");
        goto error;
    }

    varCamName=cameraName;

    if (FAILED(hr=PropBag.Write(TEXT("VCapName"),&varCamName)))
    {
        msg=TEXT("Error configuring property bag.");
        goto error;
    }

    if (FAILED(hr=pPropertyBag->Load(&PropBag,NULL)))
    {
        msg=TEXT("Error loading property bag.");
        goto error;
    }

    // create color converter
    if (FAILED(hr=pColourFilter.CoCreateInstance(CLSID_Colour)))
    {
        msg=TEXT("Error creating color converter.");
        goto error;
    }

    // create video renderer
    if (FAILED(hr=pVideoRenderer.CoCreateInstance(CLSID_VideoMixingRenderer)))
    {
        msg=TEXT("Error creating video renderer.");
        goto error;
    }

    // create the graph builder and filter graph and add video capture filter and image sink filters to it   
    if (FAILED(hr=pCaptureGraphBuilder.CoCreateInstance(CLSID_CaptureGraphBuilder)))
    {
        msg=TEXT("Error creating graph builder.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph.CoCreateInstance(CLSID_FilterGraph)))
    {
        msg=TEXT("Error creating filter graph.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph.QueryInterface(&pVideoWindow)))
    {
        msg=TEXT("Error getting video window interface.");
        goto error;
    }    

    if (FAILED(hr=pCaptureGraphBuilder->SetFiltergraph(pFilterGraph)))
    {
        msg=TEXT("Error adding filter graph to graph builder.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph->AddFilter(pVideoCaptureFilter,TEXT("Video Capture"))))
    {
        msg=TEXT("Error adding video capture filter to filter graph.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph->AddFilter(pColourFilter,TEXT("Colour converter"))))
    {
        msg=TEXT("Error adding video capture filter to filter graph.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph->AddFilter(pVideoRenderer,TEXT("Video renderer"))))
    {
        msg=TEXT("Error adding image sink to filter graph.");
        goto error;
    }
    
    // we need to configure the resolution
    if (FAILED(hr=pCaptureGraphBuilder->FindPin(pVideoCaptureFilter,PINDIR_OUTPUT,&PIN_CATEGORY_PREVIEW,&MEDIATYPE_Video,FALSE,0,&pPreviewPin)))
    {
        msg=TEXT("Error accessing preview pin.");
        goto error;
    }

    if (FAILED(hr=pCaptureGraphBuilder->FindInterface(&PIN_CATEGORY_PREVIEW, &MEDIATYPE_Video, pVideoCaptureFilter, IID_IAMStreamConfig, (void **) &pPreviewStreamConfig)))
    {
        msg=TEXT("Error accessing configuration interface.");
        goto error;
    }

	if (argc==1)
    {           
        if (FAILED(hr=EnumConfigs(pPreviewStreamConfig,NULL,ConfigComparePrint,NULL)))
        {
            msg=TEXT("Error enumerating configurations.");
            goto error;
        }
        // we just needed to print out configurations
        return S_OK;
    }    

    AM_MEDIA_TYPE* config=NULL;

    if (FAILED(hr=EnumConfigs(pPreviewStreamConfig,&config,ConfigCompareString,argv[1])))
    {
        msg=TEXT("Error enumerating configurations.");
        goto error;
    }

    if (!config)
    {
        msg=TEXT("No compatible resolution found.");
        goto error;
    }

    LONG videoWidth=abs(((VIDEOINFOHEADER*)config->pbFormat)->bmiHeader.biWidth);
    LONG videoHeight=abs(((VIDEOINFOHEADER*)config->pbFormat)->bmiHeader.biHeight);

    if (FAILED(hr=pPreviewStreamConfig->SetFormat(config)))
    {
        msg=TEXT("Can't configure selected video format.");
        goto error;
    }

    // configure the video window
    RECT screen;
    RECT video;

    GetWindowRect(GetDesktopWindow(),&screen);

    video.left=max(0,(screen.right-videoWidth)/2);
    video.top=max(0,(screen.bottom-videoHeight)/2);
    video.right=min(screen.right,(screen.right+videoWidth)/2);
    video.bottom=min(screen.bottom,(screen.bottom+videoHeight)/2);

    if ((videoWidth>screen.right)||(videoHeight>screen.bottom))
    {
        _tprintf(TEXT("Capture resolution is higher than video resolution, video will be stretched, frame rate may decrease and CPU usage will increase.\r\n"));
    }

    // renders the stream
    if (FAILED(hr=pCaptureGraphBuilder->RenderStream(&PIN_CATEGORY_PREVIEW,&MEDIATYPE_Video,pVideoCaptureFilter,pColourFilter,pVideoRenderer)))
    {
        msg=TEXT("Error rendering stream.");
        goto error;
    }
    
    if (FAILED(hr=pVideoWindow->SetWindowPosition(video.left,video.top,video.right-video.left,video.bottom-video.top)))
    {
        msg=TEXT("Can't configure video window position.");
        goto error;
    }

    DeleteMediaType(config);

    // now we need to start the graph
    if (FAILED(hr=pFilterGraph.QueryInterface(&pMediaControl)))
    {
        msg=TEXT("Error accessing media control interface.");
        goto error;
    }

    _tprintf(TEXT("Activating preview for %d seconds.\r\n"),timeout);

    if (FAILED(hr=pMediaControl->Run()))
    {
        msg=TEXT("Error running the graph.");
        goto error;
    }

    Sleep(timeout*1000);
    _tprintf(TEXT("Preview terminated.\r\n"));   
    return hr;

error:

    _tprintf(TEXT("%s\r\nCode: %d(%08x)"),msg,hr,hr);
    return hr;
}

