// stillcapture.cpp : Defines the entry point for the console application.
//

#include <windows.h>
#include <atlbase.h>
#include <streams.h>
#include <dmodshow.h>
#include <dmoreg.h>
#include <wmcodecids.h>
#include <cs.h>
#include <csmedia.h>

#include "../common/CPropertyBag.h"
#include "../common/utils.h"

int _tmain(int argc, _TCHAR* argv[])
{
    HRESULT                         hr;
    TCHAR*                          msg=NULL;
    CComPtr<ICaptureGraphBuilder2>  pCaptureGraphBuilder;
    CComPtr<IMediaEvent>            pMediaEvent;
    CComPtr<IAMVideoControl>        pVideoControl;
    CComPtr<IGraphBuilder>          pFilterGraph;
    CComPtr<IBaseFilter>            pVideoCaptureFilter;
    CComPtr<IBaseFilter>            pColourFilter;
    CComPtr<IBaseFilter>            pImageSinkFilter;
    CComPtr<IFileSinkFilter2>       pStillFileSink;
    CComPtr<IMediaControl>          pMediaControl;
    CComPtr<IPin>                   pStillPin;
    CComPtr<IAMStreamConfig>        pStillStreamConfig;
    CComPtr<IPersistPropertyBag>    pPropertyBag;
    CComVariant                     varCamName;
    CPropertyBag                    PropBag;

    if ((argc==1)||(argc>3))
    {
        _tprintf(TEXT("usage: stillcapture <res> [filename]\r\n"));
        _tprintf(TEXT("[filename] is optional and must be the full path of saved image, if no path is specified \\<res>.JPG will be used.\r\n"));
        _tprintf(TEXT("<res> is mandatory and can be one of the following values:\r\n"));
    }

    // initialize COM
    if (FAILED(hr=CoInitializeEx(NULL,COINIT_MULTITHREADED)))
    {
        msg=TEXT("CoInitialize failed.");
        goto error;
    }


    // create an instance of the video capture filter
    if (FAILED(hr=pVideoCaptureFilter.CoCreateInstance(CLSID_VideoCapture)))
    {
        msg=TEXT("Error creating capture filter.");
        goto error;
    }

    if (FAILED(hr=pVideoCaptureFilter.QueryInterface(&pVideoControl)))
    {
        msg=TEXT("Error getting capture filter control interface.");
        goto error;
    }

    // match it with a video capture driver
    if (FAILED(hr=pVideoCaptureFilter.QueryInterface(&pPropertyBag)))
    {
        msg=TEXT("Error getting capture filter interface.");
        goto error;
    }   

    TCHAR cameraName[_MAX_PATH+1];

    if (FAILED(hr=GetCameraDriver(cameraName)))
    {
        msg=TEXT("Error searching for camera driver.");
        goto error;
    }

    varCamName=cameraName;

    if (FAILED(hr=PropBag.Write(TEXT("VCapName"),&varCamName)))
    {
        msg=TEXT("Error configuring property bag.");
        goto error;
    }

    if (FAILED(hr=pPropertyBag->Load(&PropBag,NULL)))
    {
        msg=TEXT("Error loading property bag.");
        goto error;
    }

    // create color converter
    if (FAILED(hr=pColourFilter.CoCreateInstance(CLSID_Colour)))
    {
        msg=TEXT("Error creating capture filter.");
        goto error;
    }

    // create image sink
    if (FAILED(hr=pImageSinkFilter.CoCreateInstance(CLSID_IMGSinkFilter)))
    {
        msg=TEXT("Error creating image sink.");
        goto error;
    }

    if (FAILED(hr=pImageSinkFilter.QueryInterface(&pStillFileSink)))
    {
        msg=TEXT("Error getting image sink interface.");
        goto error;
    }

    if (FAILED(hr=pStillFileSink->SetMode(AM_FILE_OVERWRITE)))
    {
        msg=TEXT("Error getting image sink interface.");
        goto error;
    }

    if (argc>=3)
    {
        if (FAILED(hr=pStillFileSink->SetFileName(argv[2],NULL)))
        {
            msg=TEXT("Error configuring output filename.");
            goto error;
        }
    }
    else
    {
        if (argc>1)
        {
            // otherwise just use resolution
            TCHAR filename[_MAX_PATH+1];

            _tcsncpy(filename,TEXT("\\"),MAX_PATH);
            _tcsncat(filename,argv[1],MAX_PATH);
            _tcsncat(filename,TEXT(".png"),MAX_PATH);

            if (FAILED(hr=pStillFileSink->SetFileName(filename,NULL)))
            {
                msg=TEXT("Error configuring output filename.");
                goto error;
            }
        }
    }

    // create the graph builder and filter graph and add video capture filter and image sink filters to it   
    if (FAILED(hr=pCaptureGraphBuilder.CoCreateInstance(CLSID_CaptureGraphBuilder)))
    {
        msg=TEXT("Error creating graph builder.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph.CoCreateInstance(CLSID_FilterGraph)))
    {
        msg=TEXT("Error creating filter graph.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph.QueryInterface(&pMediaEvent)))
    {
        msg=TEXT("Error getting media event interface.");
        goto error;
    }    

    if (FAILED(hr=pCaptureGraphBuilder->SetFiltergraph(pFilterGraph)))
    {
        msg=TEXT("Error adding filter graph to graph builder.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph->AddFilter(pVideoCaptureFilter,TEXT("Video Capture"))))
    {
        msg=TEXT("Error adding video capture filter to filter graph.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph->AddFilter(pColourFilter,TEXT("Colour converter"))))
    {
        msg=TEXT("Error adding color converter filter to filter graph.");
        goto error;
    }

    if (FAILED(hr=pFilterGraph->AddFilter(pImageSinkFilter,TEXT("Image sink"))))
    {
        msg=TEXT("Error adding image sink to filter graph.");
        goto error;
    }
    
    // we need to configure the resolution
    if (FAILED(hr=pCaptureGraphBuilder->FindPin(pVideoCaptureFilter,PINDIR_OUTPUT,&PIN_CATEGORY_STILL,&MEDIATYPE_Video,FALSE,0,&pStillPin)))
    {
        msg=TEXT("Error accessing still capture pin.");
        goto error;
    }

    if (FAILED(hr=pCaptureGraphBuilder->FindInterface(&PIN_CATEGORY_STILL, &MEDIATYPE_Video, pVideoCaptureFilter, IID_IAMStreamConfig, (void **) &pStillStreamConfig)))
    {
        msg=TEXT("Error accessing configuration interface.");
        goto error;
    }

	if (argc==1)
    {           
        if (FAILED(hr=EnumConfigs(pStillStreamConfig,NULL,ConfigComparePrint,NULL)))
        {
            msg=TEXT("Error enumerating configurations.");
            goto error;
        }
        // we just needed to print out configurations
        return S_OK;
    }

    AM_MEDIA_TYPE* config=NULL;

    if (FAILED(hr=EnumConfigs(pStillStreamConfig,&config,ConfigCompareString,argv[1])))
    {
        msg=TEXT("Error enumerating configurations.");
        goto error;
    }

    if (!config)
    {
        msg=TEXT("No compatible resolution found.");
        goto error;
    }

    if (FAILED(hr=pStillStreamConfig->SetFormat(config)))
    {
        msg=TEXT("Can't configure selected video format.");
        goto error;
    }

    DeleteMediaType(config);

    // renders the stream
    if (FAILED(hr=pCaptureGraphBuilder->RenderStream(&PIN_CATEGORY_STILL,&MEDIATYPE_Video,pVideoCaptureFilter,pColourFilter,pImageSinkFilter)))
    {
        msg=TEXT("Error rendering stream.");
        goto error;
    }

    // now we need to start the graph
    if (FAILED(hr=pFilterGraph.QueryInterface(&pMediaControl)))
    {
        msg=TEXT("Error accessing media control interface.");
        goto error;
    }

    if (FAILED(hr=pMediaControl->Run()))
    {
        msg=TEXT("Error running the graph.");
        goto error;
    }

    if (FAILED(hr=pVideoControl->SetMode(pStillPin,VideoControlFlag_Trigger)))
    {
        msg=TEXT("Error triggering capture event.");
        goto error;
    }

    long code,param1,param2;
    DWORD start=GetTickCount();

    do
    {
        if (FAILED(hr=pMediaEvent->GetEvent(&code,&param1,&param2,10000)))
        {
            msg=TEXT("Error while waiting for event.");
            goto error;
        }

        if (FAILED(hr=pMediaEvent->FreeEventParams(code,param1,param2)))
        {
            msg=TEXT("Error freeing event information.");
            goto error;
        }        
    }
    while ((code!=EC_CAP_FILE_COMPLETED)&&(GetTickCount()-start<10000));

    if (code!=EC_CAP_FILE_COMPLETED)
    {
        _tprintf(TEXT("No image has been captured."));
    }
    else
    {
        _tprintf(TEXT("Still image captured.\r\n"));
    }
   
    return hr;

error:

    _tprintf(TEXT("%s\r\nCode: %d(%08x)"),msg,hr,hr);
    return hr;
}

